from .. import auth

token = "eyJhbGciOiJFZERTQSIsImtpZCI6InNpZy0xNjQ0OTM3MzYwIn0.eyJqdGkiOiI1YWJhOGJhNi05MGQ1LTQ4N2QtOWEyNy1iYzA0N2U1NTdjYzAiLCJhdWQiOlsiYXBwOjI4NDYzNjA3NjI0MjM3MTA4NSIsImh0dHBzOi8vYXBpLnJvd25kLmlvIl0sInN1YiI6Imdvb2dsZS1vYXV0aDJ8MTE0ODk1MTIyNzk1NDUyMTI3Mjc2IiwiaWF0IjoxNjQ5MzA0NjY5LCJodHRwczovL2F1dGgucm93bmQuaW8vYXBwX3VzZXJfaWQiOiIxN2E4MjE3Zi04MmUzLTQwZGItODU1Ni0xNmIxMGQzNThkYTgiLCJodHRwczovL2F1dGgucm93bmQuaW8vaXNfdmVyaWZpZWRfdXNlciI6dHJ1ZSwiaXNzIjoiaHR0cHM6Ly9hcGkucm93bmQuaW8iLCJleHAiOjE2NDkzMDgyNjl9.qct6lD7AJW97gjYCYyfvyArbVlE8jf76joCHcEbEgNkE1d-EPSgBIvA9MoRSP2Ezm4-uymCHBC7cenlObXBZBw"
auth.client.validate_jwt(token)